@@ -78,6 +78,12 @@ template<typename T, typename M> std::vector<T> model_subchains(M* model) {
 // UnknownAny = any format (coordinate file for a monomer/ligand/chemcomp)
 enum class CoorFormat { Unknown, UnknownAny, Pdb, Mmcif, Mmjson, ChemComp };
 
+// options affecting how pdb file is read
+struct PdbReadOptions {
+  int max_line_length = 0;
+  bool split_chain_on_ter = false;
+};
+
 enum class EntityType : unsigned char {
   Unknown,
   Polymer,
@@ -149,38 +155,41 @@ struct Atom {
   int serial = 0;
   Position pos;
   float occ = 1.0f;
+  // ADP - in MX it's usual to give isotropic ADP as B and anisotropic as U
   float b_iso = 20.0f; // arbitrary default value
-  float u11=0, u22=0, u33=0, u12=0, u13=0, u23=0;
+  SMat33<float> aniso = {0, 0, 0, 0, 0, 0};
 
   char altloc_or(char null_char) const { return altloc ? altloc : null_char; }
   bool same_conformer(const Atom& other) const {
     return is_same_conformer(altloc, other.altloc);
   }
-  // same_group() is for use in UniqIter
-  bool same_group(const Atom& other) const { return name == other.name; }
+  // group_key() is used in UniqIter and similar tools
+  const std::string& group_key() const { return name; }
   bool has_altloc() const { return altloc != '\0'; }
-  bool has_anisou() const {
-    return u11 != 0.f || u22 != 0.f || u33 != 0.f ||
-           u12 != 0.f || u13 != 0.f || u23 != 0.f;
-  }
-  double b_iso_from_aniso() const {
-    return 8 * pi() * pi() / 3. * (u11 + u22 + u33);
+  double b_eq() const {
+    return 8 * pi() * pi() / 3. * aniso.trace();
   }
   bool is_hydrogen() const { return gemmi::is_hydrogen(element); }
+  // a method present in Atom, Residue, ... Structure - used in templates
+  Atom empty_copy() const { return Atom(*this); }
 };
 
-struct AtomGroup : MutableVectorSpan<Atom> {
-  using MutableVectorSpan::MutableVectorSpan;
-  std::string name() const { return size() != 0 ? front().name : ""; }
-  Atom& by_altloc(char alt) { return impl::get_by_altloc(*this, alt); }
+template<typename AtomType>
+struct AtomGroup_ : ItemGroup<AtomType> {
+  using ItemGroup<AtomType>::ItemGroup;
+  std::string name() const { return !this->empty() ? this->front().name : ""; }
+  AtomType& by_altloc(char alt) {
+    for (int i = 0; i != this->extent(); ++i) {
+      AtomType* a = &this->front() + i;
+      if (a->altloc == alt && (a->name == this->front().name))
+        return *a;
+    }
+    fail("No such altloc");
+  }
 };
 
-struct ConstAtomGroup : Span<const Atom> {
-  ConstAtomGroup(const Atom* begin, size_t n) : Span(begin, n) {}
-  ConstAtomGroup(const AtomGroup& o) : Span(o.begin(), o.size()) {}
-  std::string name() const { return size() != 0 ? front().name : ""; }
-  const Atom& by_altloc(char a) const { return impl::get_by_altloc(*this, a); }
-};
+using AtomGroup = AtomGroup_<Atom>;
+using ConstAtomGroup = AtomGroup_<const Atom>;
 
 // Sequence ID (sequence number + insertion code) + residue name + segment ID
 struct ResidueId {
@@ -188,8 +197,8 @@ struct ResidueId {
   std::string segment; // segid - up to 4 characters in the PDB file
   std::string name;
 
-  // checks for equality of sequence ID; used for first_conformation iterators
-  bool same_group(const ResidueId& o) const { return seqid == o.seqid; }
+  // used for first_conformation iterators, etc.
+  SeqId group_key() const { return seqid; }
 
   bool matches(const ResidueId& o) const {
     return seqid == o.seqid && segment == o.segment && name == o.name;
@@ -206,11 +215,23 @@ struct Residue : public ResidueId {
   EntityType entity_type = EntityType::Unknown;
   char het_flag = '\0';   // 'A' = ATOM, 'H' = HETATM, 0 = unspecified
   bool is_cis = false;    // bond to the next residue marked as cis
+  char flag = '\0';       // custom flag
   std::vector<Atom> atoms;
 
   Residue() = default;
   explicit Residue(const ResidueId& rid) noexcept : ResidueId(rid) {}
 
+  // copy all but atoms (children) - for use in templates
+  Residue empty_copy() const {
+    Residue res((ResidueId&)*this);
+    res.subchain = subchain;
+    res.label_seq = label_seq;
+    res.entity_type = entity_type;
+    res.het_flag = het_flag;
+    res.is_cis = is_cis;
+    res.flag = flag;
+    return res;
+  }
   std::vector<Atom>& children() { return atoms; }
   const std::vector<Atom>& children() const { return atoms; }
 
@@ -244,21 +265,17 @@ struct Residue : public ResidueId {
   }
 
   AtomGroup get(const std::string& atom_name) {
-    auto func = [&](const Atom& a) { return a.name == atom_name; };
-    auto g_begin = std::find_if(atoms.begin(), atoms.end(), func);
-    if (g_begin == atoms.end())
-      fail("No such atom: " + atom_name);
-    auto g_end = std::find_if_not(g_begin, atoms.end(), func);
-    if (std::find_if(g_end, atoms.end(), func) != atoms.end())
-      fail("Non-consecutive alternative location of the atom");
-    return AtomGroup(atoms, &*g_begin, g_end - g_begin);
+    for (Atom& atom : atoms)
+      if (atom.name == atom_name)
+        return AtomGroup(&atom, atoms.data() + atoms.size());
+    fail("No such atom: " + atom_name);
   }
 
   Atom& sole_atom(const std::string& atom_name) {
     AtomGroup aa = get(atom_name);
     if (aa.size() != 1)
       fail("Multiple alternative atoms " + atom_name);
-    return aa[0];
+    return aa.front();
   }
 
   // short-cuts to access peptide backbone atoms
@@ -323,7 +340,7 @@ struct ConstResidueSpan : Span<const Residue> {
   int length() const {
     int length = (int) size();
     for (int n = length - 1; n > 0; --n)
-      if ((begin() + n)->same_group(*(begin() + n - 1)))
+      if ((begin() + n)->group_key() == (begin() + n - 1)->group_key())
         --length;
     return length;
   }
@@ -451,6 +468,12 @@ struct ConstResidueGroup : ConstResidueSpan {
 inline ResidueGroup ResidueSpan::find_residue_group(SeqId id) {
   return ResidueSpan(subspan([&](const Residue& r) { return r.seqid == id; }));
 }
+inline ConstResidueGroup ResidueSpan::find_residue_group(SeqId id) const {
+  return const_().find_residue_group(id);
+}
+inline ConstResidueGroup ConstResidueSpan::find_residue_group(SeqId id) const {
+  return ConstResidueSpan(subspan([&](const Residue& r) { return r.seqid == id; }));
+}
 
 struct ResidueSpan::GroupingProxy {
   ResidueSpan& span;
@@ -560,12 +583,15 @@ struct Chain {
 
   Residue* find_or_add_residue(const ResidueId& rid);
   void append_residues(std::vector<Residue> new_resi, int min_sep=0);
+
+  // methods present in Structure, Model, ... - used in templates
+  Chain empty_copy() const { return Chain(name); }
   std::vector<Residue>& children() { return residues; }
   const std::vector<Residue>& children() const { return residues; }
 
   // Returns false only for alternative conformation (microheterogeneity).
   bool is_first_in_group(const Residue& res) const {
-    return &res == residues.data() || !(&res - 1)->same_group(res);
+    return &res == residues.data() || (&res - 1)->group_key() != res.group_key();
   }
 
   // Returns the previous residue or nullptr.
@@ -573,8 +599,8 @@ struct Chain {
   const Residue* previous_residue(const Residue& res) const {
     const Residue* start = residues.data();
     for (const Residue* p = &res; p != start; )
-      if (!res.same_group(*--p)) {
-        while (p != start && p->same_group(*(p-1)) &&
+      if (res.group_key() != (--p)->group_key()) {
+        while (p != start && p->group_key() == (p-1)->group_key() &&
                (res.atoms.at(0).altloc == '\0' || !res.same_conformer(*p)))
           --p;
         return p;
@@ -586,8 +612,9 @@ struct Chain {
   const Residue* next_residue(const Residue& res) const {
     const Residue* end = residues.data() + residues.size();
     for (const Residue* p = &res + 1; p != end; ++p)
-      if (!res.same_group(*p)) {
-        while (p+1 != end && p->same_group(*(p+1)) && !res.same_conformer(*p))
+      if (res.group_key() != p->group_key()) {
+        while (p+1 != end && p->group_key() == (p+1)->group_key() &&
+               !res.same_conformer(*p))
           ++p;
         return p;
       }
@@ -693,10 +720,10 @@ inline bool atom_matches(const const_CRA& cra, const AtomAddress& addr) {
 // Symmetry operators are not trusted and not stored.
 // We assume that the nearest symmetry mate is connected.
 struct Connection {
-  enum Type { Covale, Disulf, Hydrog, MetalC, None };
+  enum Type { Covale, Disulf, Hydrog, MetalC, Unknown };
   std::string name;
   std::string link_id;  // _struct_conn.ccp4_link_id (== _chem_link.id)
-  Type type = None;
+  Type type = Unknown;
   Asu asu = Asu::Any;
   AtomAddress partner1, partner2;
   double reported_distance = 0.0;
@@ -759,8 +786,7 @@ class CraIterPolicy {
 public:
   typedef CraT value_type;
   CraIterPolicy() : chains_end(nullptr), cra{nullptr, nullptr, nullptr} {}
-  CraIterPolicy(const Chain* end, CraT cra_)
-    : chains_end(end), cra(cra_) {}
+  CraIterPolicy(const Chain* end, CraT cra_) : chains_end(end), cra(cra_) {}
   void increment() {
     if (cra.atom)
       if (++cra.atom == vector_end_ptr(cra.residue->atoms)) {
@@ -791,18 +817,23 @@ private:
   CraT cra;
 };
 
-struct ConstCraProxy {
-  const std::vector<Chain>& chains;
-  using iterator = BidirIterator<CraIterPolicy<const_CRA>>;
+template<typename CraT, typename ChainsRefT>
+struct CraProxy_ {
+  ChainsRefT chains;
+  using iterator = BidirIterator<CraIterPolicy<CraT>>;
   iterator begin() {
-    auto chain = &chains.at(0);
-    auto residue = &chain->residues.at(0);
-    auto atom = &residue->atoms.at(0);
-    return CraIterPolicy<const_CRA>{vector_end_ptr(chains), {chain, residue, atom}};
+    CraT cra;
+    cra.chain = &chains.at(0);
+    cra.residue = &cra.chain->residues.at(0);
+    cra.atom = &cra.residue->atoms.at(0);
+    return CraIterPolicy<CraT>{vector_end_ptr(chains), cra};
   }
   iterator end() { return {}; }
 };
 
+using CraProxy = CraProxy_<CRA, std::vector<Chain>&>;
+using ConstCraProxy = CraProxy_<const_CRA, const std::vector<Chain>&>;
+
 struct Model {
   static const char* what() { return "Model"; }
   std::string name;  // actually an integer number
@@ -917,6 +948,7 @@ struct Model {
     return const_cast<Model*>(this)->find_cra(address);
   }
 
+  CraProxy all() { return {chains}; }
   ConstCraProxy all() const { return {chains}; }
 
   Atom* find_atom(const AtomAddress& address) { return find_cra(address).atom; }
@@ -937,6 +969,8 @@ struct Model {
     return table;
   }
 
+  // methods present in Structure, Model, ... - used in templates
+  Model empty_copy() const { return Model(name); }
   std::vector<Chain>& children() { return chains; }
   const std::vector<Chain>& children() const { return chains; }
 };
@@ -977,14 +1011,18 @@ struct Assembly {
   Assembly(const std::string& name_) : name(name_) {}
 };
 
-inline const Entity* get_entity_of(const ConstResidueSpan& sub,
-                                   const std::vector<Entity>& entities) {
-  if (sub && !sub.subchain_id().empty())
-    for (const Entity& ent : entities)
-      if (in_vector(sub.subchain_id(), ent.subchains))
+inline Entity* find_entity(const std::string& subchain_id,
+                           std::vector<Entity>& entities) {
+  if (!subchain_id.empty())
+    for (Entity& ent : entities)
+      if (in_vector(subchain_id, ent.subchains))
         return &ent;
   return nullptr;
 }
+inline const Entity* find_entity(const std::string& subchain_id,
+                                 const std::vector<Entity>& entities) {
+  return find_entity(subchain_id, const_cast<std::vector<Entity>&>(entities));
+}
 
 struct Structure {
   std::string name;
@@ -1054,11 +1092,11 @@ struct Structure {
     return const_cast<Structure*>(this)->get_entity(ent_id);
   }
 
-  const Entity* get_entity_of(const ConstResidueSpan& sub) const {
-    return gemmi::get_entity_of(sub, entities);
-  }
   Entity* get_entity_of(const ConstResidueSpan& sub) {
-    return const_cast<Entity*>(gemmi::get_entity_of(sub, entities));
+    return sub ? find_entity(sub.subchain_id(), entities) : nullptr;
+  }
+  const Entity* get_entity_of(const ConstResidueSpan& sub) const {
+    return const_cast<Structure*>(this)->get_entity_of(sub);
   }
 
   Assembly* find_assembly(const std::string& assembly_id) {
@@ -1097,8 +1135,30 @@ struct Structure {
       model.merge_chain_parts(min_sep);
   }
 
+  // copy all but models (in general, empty_copy copies all but children)
+  Structure empty_copy() const {
+    Structure st;
+    st.name = name;
+    st.cell = cell;
+    st.spacegroup_hm = spacegroup_hm;
+    st.ncs = ncs;
+    st.entities = entities;
+    st.connections = connections;
+    st.helices = helices;
+    st.sheets = sheets;
+    st.assemblies = assemblies;
+    st.meta = meta;
+    st.has_origx = has_origx;
+    st.origx = origx;
+    st.info = info;
+    st.raw_remarks = raw_remarks;
+    st.resolution = resolution;
+    st.input_format = input_format;
+    return st;
+  }
   std::vector<Model>& children() { return models; }
   const std::vector<Model>& children() const { return models; }
+
   void setup_cell_images();
 };
 
