@@ -22,26 +22,42 @@
 #include "symmetry.hpp"  // for find_spacegroup_by_name, SpaceGroup
 #include "unitcell.hpp"  // for UnitCell
 #include "util.hpp"      // for ialpha4_id, rtrim_str, ialpha3_id, ...
+#include "reciproc.hpp"  // for calculate_min_max_disregarding_nans
 
 namespace gemmi {
 
-template <typename T, typename FP=typename std::iterator_traits<T>::value_type>
-std::array<FP,2> calculate_min_max_disregarding_nans(T begin, T end) {
-  std::array<FP,2> minmax = {{NAN, NAN}};
-  T i = begin;
-  while (i != end && std::isnan(*i))
-    ++i;
-  if (i != end) {
-    minmax[0] = minmax[1] = *i;
-    while (++i != end) {
-      if (*i < minmax[0])
-        minmax[0] = *i;
-      else if (*i > minmax[1])
-        minmax[1] = *i;
-    }
+// Unmerged MTZ files always store in-asu hkl indices and symmetry operation
+// encoded in the M/ISYM column. Here is a helper for writing such files.
+struct UnmergedHklMover {
+  UnmergedHklMover(const SpaceGroup* spacegroup) : asu_(spacegroup) {
+    if (spacegroup)
+      group_ops_ = spacegroup->operations();
   }
-  return minmax;
-}
+
+  // Modifies hkl and returns ISYM value for M/ISYM
+  int move_to_asu(std::array<int, 3>& hkl) {
+    int isym = 1;
+    for (Op op : group_ops_) {
+      Miller new_hkl = op.apply_to_hkl(hkl);
+      if (asu_.is_in(new_hkl)) {
+        hkl = new_hkl;
+        return isym;
+      }
+      Miller negated_new_hkl{{-new_hkl[0], -new_hkl[1], -new_hkl[2]}};
+      if (asu_.is_in(negated_new_hkl)) {
+        hkl = negated_new_hkl;
+        return isym + 1;
+      }
+      isym += 2;
+    }
+    return 0;
+  }
+
+private:
+  ReciprocalAsu asu_;
+  GroupOps group_ops_;
+};
+
 
 struct Mtz {
   struct Dataset {
@@ -67,7 +83,7 @@ struct Mtz {
     const Dataset& dataset() const { return parent->dataset(dataset_id); }
     bool has_data() const { return parent->has_data(); }
     int size() const { return has_data() ? parent->nreflections : 0; }
-    unsigned stride() const { return (unsigned) parent->columns.size(); }
+    size_t stride() const { return parent->columns.size(); }
     float& operator[](int n) { return parent->data[idx + n * stride()]; }
     float operator[](int n) const { return parent->data[idx + n * stride()]; }
     float& at(int n) { return parent->data.at(idx + n * stride()); }
@@ -106,6 +122,10 @@ struct Mtz {
     std::vector<float> floats;
     std::vector<std::string> axes;
 
+    UnitCell get_cell() const {
+      return UnitCell(floats[0], floats[1], floats[2],
+                      floats[3], floats[4], floats[5]);
+    }
     void set_cell(const UnitCell& uc) {
       floats[0] = (float) uc.a;
       floats[1] = (float) uc.b;
@@ -114,6 +134,15 @@ struct Mtz {
       floats[4] = (float) uc.beta;
       floats[5] = (float) uc.gamma;
     }
+
+    int dataset_id() const { return ints[20]; }
+    float phi_start() const { return floats[36]; }
+    float phi_end() const { return floats[37]; }
+    Mat33 matrix_U() const {
+      return Mat33(floats[6], floats[9],  floats[12],
+                   floats[7], floats[10], floats[13],
+                   floats[8], floats[11], floats[14]);
+    }
   };
 
   bool same_byte_order = true;
@@ -139,7 +168,13 @@ struct Mtz {
 
   FILE* warnings = nullptr;
 
-  Mtz() = default;
+  explicit Mtz(bool with_base=false) {
+    if (with_base) {
+      datasets.push_back({0, "HKL_base", "HKL_base", "HKL_base", cell, 0.});
+      for (int i = 0; i != 3; ++i)
+        add_column(std::string(1, "HKL"[i]), 'H', 0, i);
+    }
+  }
   Mtz(Mtz&& o) noexcept { *this = std::move(o); }
   Mtz& operator=(Mtz&& o) noexcept {
     same_byte_order = o.same_byte_order;
@@ -186,6 +221,13 @@ struct Mtz {
     return const_cast<Mtz*>(this)->get_cell(dataset);
   }
 
+  void set_cell_for_all(const UnitCell& new_cell) {
+    cell = new_cell;
+    cell.set_cell_images_from_spacegroup(spacegroup);  // probably not needed
+    for (Dataset& ds : datasets)
+      ds.cell = cell;
+  }
+
   Dataset& last_dataset() {
     if (datasets.empty())
       fail("MTZ dataset not found (missing DATASET header line?).");
@@ -249,7 +291,7 @@ struct Mtz {
 
   void extend_min_max_1_d2(const UnitCell& uc, double& min, double& max) const {
     for (size_t i = 0; i < data.size(); i += columns.size()) {
-      double res = uc.calculate_1_d2(data[i+0], data[i+1], data[i+2]);
+      double res = uc.calculate_1_d2_double(data[i+0], data[i+1], data[i+2]);
       if (res < min)
         min = res;
       if (res > max)
@@ -276,6 +318,12 @@ struct Mtz {
     return {{min_value, max_value}};
   }
 
+  void update_reso() {
+    std::array<double,2> reso = calculate_min_max_1_d2();
+    min_1_d2 = reso[0];
+    max_1_d2 = reso[1];
+  }
+
   // Functions for reading MTZ headers and data.
 
   void toggle_endiannes() {
@@ -325,7 +373,7 @@ struct Mtz {
     return UnitCell(a, b, c, alpha, beta, gamma);
   }
 
-  void warn(const std::string& text) {
+  void warn(const std::string& text) const {
     if (warnings)
       std::fprintf(warnings, "%s\n", text.c_str());
   }
@@ -343,6 +391,7 @@ struct Mtz {
     char line[81] = {0};
     seek_headers(stream);
     int ncol = 0;
+    bool has_batch = false;
     while (stream.read(line, 80) && ialpha3_id(line) != ialpha3_id("END")) {
       const char* args = skip_word(line);
       switch (ialpha4_id(line)) {
@@ -455,7 +504,9 @@ struct Mtz {
             warn("MTZ DWAV line: unusual numbering.");
           break;
         case ialpha4_id("BATCH"):
-          // this header is not used for anything?
+          // We take number of batches from the NCOL record and serial numbers
+          // from BH. This header could be used only to check consistency.
+          has_batch = true;
           break;
         default:
           warn("Unknown header: " + rtrim_str(line));
@@ -463,6 +514,8 @@ struct Mtz {
     }
     if (ncol != (int) columns.size())
       fail("Number of COLU records inconsistent with NCOL record.");
+    if (has_batch != !batches.empty())
+      fail("BATCH header inconsistent with NCOL record.");
   }
 
   // read the part between END and MTZENDOFHEADERS
@@ -505,7 +558,7 @@ struct Mtz {
           stream.read(buf, 80);
           if (ialpha4_id(buf) != ialpha4_id("BHCH"))
             fail("Missing BHCH header");
-          split_str_into(buf + 5, ' ', batch.axes);
+          split_str_into_multi(buf + 5, " \t", batch.axes);
         }
       }
     }
@@ -544,6 +597,8 @@ struct Mtz {
     read_main_headers(stream);
     read_history_and_batch_headers(stream);
     setup_spacegroup();
+    if (datasets.empty())
+      datasets.push_back({0, "HKL_base", "HKL_base", "HKL_base", cell, 0.});
   }
 
   template<typename Stream>
@@ -591,13 +646,21 @@ struct Mtz {
     return indices;
   }
 
-  // Change HKL according to M/ISYM
-  void apply_isym() {
+  Miller get_hkl(size_t offset) const {
+    return {{(int)data[offset], (int)data[offset+1], (int)data[offset+2]}};
+  }
+  void set_hkl(size_t offset, const Miller& hkl) {
+    for (int i = 0; i != 3; ++i)
+      data[offset + i] = static_cast<float>(hkl[i]);
+  }
+
+  // (for unmerged MTZ only) change HKL according to M/ISYM
+  bool switch_to_original_hkl() {
     if (!has_data())
-      fail("apply_isym(): data not read yet");
+      fail("switch_to_original_hkl(): data not read yet");
     const Column* col = column_with_label("M/ISYM");
     if (col == nullptr || col->type != 'Y' || col->idx < 3)
-      return;
+      return false;
     std::vector<Op> inv_symops;
     inv_symops.reserve(symops.size());
     for (const Op& op : symops)
@@ -605,12 +668,31 @@ struct Mtz {
     for (size_t n = 0; n + col->idx < data.size(); n += columns.size()) {
       int isym = static_cast<int>(data[n + col->idx]) & 0xFF;
       const Op& op = inv_symops.at((isym - 1) / 2);
-      std::array<int,3> hkl = {{(int)data[n], (int)data[n+1], (int)data[n+2]}};
-      hkl = op.apply_to_hkl(hkl);
+      Miller hkl = op.apply_to_hkl(get_hkl(n));
       int sign = (isym & 1) ? 1 : -1;
       for (int i = 0; i < 3; ++i)
         data[n+i] = static_cast<float>(sign * hkl[i]);
     }
+    return true;
+  }
+
+  // (for unmerged MTZ only) change HKL to ASU equivalent and set ISYM
+  bool switch_to_asu_hkl() {
+    if (!has_data())
+      fail("switch_to_asu_hkl(): data not read yet");
+    const Column* col = column_with_label("M/ISYM");
+    if (col == nullptr || col->type != 'Y' || col->idx < 3 || !spacegroup)
+      return false;
+    size_t misym_idx = col->idx;
+    UnmergedHklMover hkl_mover(spacegroup);
+    for (size_t n = 0; n + col->idx < data.size(); n += columns.size()) {
+      Miller hkl = get_hkl(n);
+      int isym = hkl_mover.move_to_asu(hkl);  // modifies hkl
+      set_hkl(n, hkl);
+      float& misym = data[n + misym_idx];
+      misym = float(((int)misym & ~0xff) | isym);
+    }
+    return true;
   }
 
   Dataset& add_dataset(const std::string& name) {
@@ -623,7 +705,7 @@ struct Mtz {
   }
 
   Column& add_column(const std::string& label, char type,
-                     int dataset_id=-1, int pos=-1) {
+                     int dataset_id=-1, int pos=-1, bool expand_data=false) {
     if (datasets.empty())
       fail("No datasets.");
     if (dataset_id < 0)
@@ -642,6 +724,8 @@ struct Mtz {
     col->label = label;
     col->parent = this;
     col->idx = pos;
+    if (expand_data)
+      expand_data_rows(1);
     return *col;
   }
 
@@ -671,38 +755,6 @@ struct Mtz {
   void write_to_file(const std::string& path) const;
 };
 
-// Unmerged MTZ files always store in-asu hkl indices and symmetry operation
-// encoded in the M/ISYM column. Here is a helper for writing such files.
-struct UnmergedHklMover {
-  UnmergedHklMover(const Mtz& mtz)
-    : asu_checker_(mtz.spacegroup),
-      group_ops_(mtz.spacegroup->operations())
-  {}
-
-  // Modifies hkl and returns ISYM value for M/ISYM
-  int move_to_asu(std::array<int, 3>& hkl) {
-    int isym = 1;
-    for (Op op : group_ops_) {
-      Miller new_hkl = op.apply_to_hkl(hkl);
-      if (asu_checker_.is_in(new_hkl)) {
-        hkl = new_hkl;
-        return isym;
-      }
-      Miller negated_new_hkl{{-new_hkl[0], -new_hkl[1], -new_hkl[2]}};
-      if (asu_checker_.is_in(negated_new_hkl)) {
-        hkl = negated_new_hkl;
-        return isym + 1;
-      }
-      isym += 2;
-    }
-    return 0;
-  }
-
-private:
-  ReciprocalAsuChecker asu_checker_;
-  GroupOps group_ops_;
-};
-
 
 inline Mtz read_mtz_file(const std::string& path) {
   Mtz mtz;
@@ -721,31 +773,26 @@ Mtz read_mtz(Input&& input, bool with_data) {
 // Abstraction of data source, cf. ReflnDataProxy.
 struct MtzDataProxy {
   const Mtz& mtz_;
-  bool ok() const { return mtz_.has_data(); }
-  constexpr std::array<size_t,3> hkl_col() const { return {{0, 1, 2}}; }
   size_t stride() const { return mtz_.columns.size(); }
   size_t size() const { return mtz_.data.size(); }
-  int get_int(size_t n) const { return (int) mtz_.data[n]; }
   float get_num(size_t n) const { return mtz_.data[n]; }
   const UnitCell& unit_cell() const { return mtz_.cell; }
   const SpaceGroup* spacegroup() const { return mtz_.spacegroup; }
-  Miller get_hkl(size_t offset, const std::array<size_t,3>&) const {
-    return {{get_int(offset + 0), get_int(offset + 1), get_int(offset + 2)}};
-  }
+  Miller get_hkl(size_t offset) const { return mtz_.get_hkl(offset); }
 };
 
-// Like above, but here the data is store outside of the Mtz class
+// Like above, but here the data is stored outside of the Mtz class
 struct MtzExternalDataProxy : MtzDataProxy {
   const float* data_;
-
   MtzExternalDataProxy(const Mtz& mtz, const float* data)
     : MtzDataProxy{mtz}, data_(data) {}
-  bool ok() const { return true; }
   size_t size() const { return mtz_.columns.size() * mtz_.nreflections; }
-  int get_int(size_t n) const { return (int) data_[n]; }
   float get_num(size_t n) const { return data_[n]; }
-private:
-  void get_hkl() const {} // disable for now
+  Miller get_hkl(size_t offset) const {
+    return {{(int)data_[offset + 0],
+             (int)data_[offset + 1],
+             (int)data_[offset + 2]}};
+  }
 };
 
 } // namespace gemmi
@@ -811,7 +858,7 @@ void Mtz::write_to_stream(std::FILE* stream) const {
     WRITE("VALM %f", valm);
   for (const Column& col : columns) {
     auto minmax = calculate_min_max_disregarding_nans(col.begin(), col.end());
-    WRITE("COLUMN %-30s %c %17.9g %17.9g %4d",
+    WRITE("COLUMN %-30s %c %17.9f %17.9f %4d",
           col.label.c_str(), col.type, minmax[0], minmax[1], col.dataset_id);
     if (!col.source.empty())
       WRITE("COLSRC %-30s %-36s  %4d",
@@ -822,9 +869,9 @@ void Mtz::write_to_stream(std::FILE* stream) const {
     WRITE("PROJECT %7d %s", ds.id, ds.project_name.c_str());
     WRITE("CRYSTAL %7d %s", ds.id, ds.crystal_name.c_str());
     WRITE("DATASET %7d %s", ds.id, ds.dataset_name.c_str());
+    const UnitCell& uc = (ds.cell.is_crystal() && ds.cell.a > 0 ? ds.cell : cell);
     WRITE("DCELL %9d %10.4f%10.4f%10.4f%10.4f%10.4f%10.4f",
-          ds.id, ds.cell.a, ds.cell.b, ds.cell.c,
-          ds.cell.alpha, ds.cell.beta, ds.cell.gamma);
+          ds.id, uc.a, uc.b, uc.c, uc.alpha, uc.beta, uc.gamma);
     WRITE("DWAVEL %8d %10.5f", ds.id, ds.wavelength);
     for (size_t i = 0; i < batches.size(); i += 12) {
       std::memcpy(buf, "BATCH ", 6);
