@@ -66,13 +66,13 @@ py::array_t<float> make_new_column(const Mtz& mtz, int dataset, F f) {
 static py::array_t<float> make_1_d2_array(const Mtz& mtz, int dataset) {
   return make_new_column(mtz, dataset,
                          [](const UnitCell& cell, float h, float k, float l) {
-                           return (float) cell.calculate_1_d2(h, k, l);
+                           return (float) cell.calculate_1_d2_double(h, k, l);
                          });
 }
 static py::array_t<float> make_d_array(const Mtz& mtz, int dataset) {
   return make_new_column(mtz, dataset,
                          [](const UnitCell& cell, float h, float k, float l) {
-                           double _1_d2 = cell.calculate_1_d2(h, k, l);
+                           double _1_d2 = cell.calculate_1_d2_double(h, k, l);
                            return (float) (1.0 / std::sqrt(_1_d2));
                          });
 }
@@ -98,7 +98,8 @@ void add_hkl(py::module& m) {
   py::bind_vector<std::vector<ReflnBlock>>(m, "ReflnBlocks");
   py::bind_vector<std::vector<const Mtz::Column*>>(m, "MtzColumnRefs");
 
-  mtz.def(py::init<>())
+  mtz
+    .def(py::init<bool>(), py::arg("with_base")=false)
     .def_buffer([](Mtz &self) {
       int nrow = self.has_data() ? self.nreflections : 0;
       int ncol = (int) self.columns.size();
@@ -139,6 +140,16 @@ void add_hkl(py::module& m) {
     })
     .def("get_cell", (UnitCell& (Mtz::*)(int)) &Mtz::get_cell,
          py::arg("dataset")=-1)
+    .def("set_cell_for_all", &Mtz::set_cell_for_all)
+    .def("make_miller_array", [](const Mtz& self) {
+        py::array_t<int> arr({self.nreflections, 3});
+        py::buffer_info buf = arr.request();
+        int* ptr = (int*) buf.ptr;
+        for (int i = 0; i < self.nreflections; ++i)
+          for (int j = 0; j != 3; ++j)
+            ptr[3*i + j] = (int) self.data[self.columns.size() * i + j];
+        return arr;
+    })
     .def("make_1_d2_array", &make_1_d2_array, py::arg("dataset")=-1)
     .def("make_d_array", &make_d_array, py::arg("dataset")=-1)
     .def("get_size_for_hkl",
@@ -196,8 +207,15 @@ void add_hkl(py::module& m) {
     .def("add_dataset", &Mtz::add_dataset, py::arg("name"),
          py::return_value_policy::reference_internal)
     .def("add_column", &Mtz::add_column, py::arg("label"), py::arg("type"),
-         py::arg("dataset_id")=-1, py::arg("pos")=-1,
+         py::arg("dataset_id")=-1, py::arg("pos")=-1, py::arg("expand_data")=true,
          py::return_value_policy::reference_internal)
+    .def("set_data", [](Mtz& self, const FPhiGrid<float>::AsuData& asu_data) {
+         if (self.columns.size() != 5)
+           fail("Mtz.set_data(): Mtz must have 5 columns to put H,K,L,F,Phi.");
+         self.nreflections = (int) asu_data.v.size();
+         self.data.clear();
+         add_asu_f_phi_to_float_vector(self.data, asu_data);
+    }, py::arg("asu_data"))
     .def("set_data", [](Mtz& self, py::array_t<float> arr) {
          if (arr.ndim() != 2)
            fail("Mtz.set_data(): expected 2D array.");
@@ -291,8 +309,8 @@ void add_hkl(py::module& m) {
     }, py::arg("tag"), py::arg("null")=NAN)
     .def("make_float_array", &ReflnBlock::make_vector<double>,
          py::arg("tag"), py::arg("null")=NAN)
-    .def("make_index_array", [](ReflnBlock& self) {
-        return py::array(py::cast((self.make_index_vector())));
+    .def("make_miller_array", [](ReflnBlock& self) {
+        return py::array_t<int>(py::cast(self.make_miller_vector()));
     })
     .def("make_1_d2_array", [](ReflnBlock& self) {
         return py_array_from_vector(self.make_1_d2_vector());
@@ -303,11 +321,11 @@ void add_hkl(py::module& m) {
     .def("get_size_for_hkl",
          [](const ReflnBlock& self,
             std::array<int,3> min_size, double sample_rate) {
-          return get_size_for_hkl(ReflnDataProxy{self}, min_size, sample_rate);
+          return get_size_for_hkl(ReflnDataProxy(self), min_size, sample_rate);
     }, py::arg("min_size")=std::array<int,3>{{0,0,0}},
        py::arg("sample_rate")=0.)
     .def("data_fits_into", [](const ReflnBlock& self, std::array<int,3> size) {
-        return data_fits_into(ReflnDataProxy{self}, size);
+        return data_fits_into(ReflnDataProxy(self), size);
     }, py::arg("size"))
     .def("get_f_phi_on_grid", [](const ReflnBlock& self,
                                  const std::string& f_col,
@@ -316,7 +334,7 @@ void add_hkl(py::module& m) {
                                  bool half_l, AxisOrder order) {
         size_t f_idx = self.get_column_index(f_col);
         size_t phi_idx = self.get_column_index(phi_col);
-        return get_f_phi_on_grid<float>(ReflnDataProxy{self}, f_idx, phi_idx,
+        return get_f_phi_on_grid<float>(ReflnDataProxy(self), f_idx, phi_idx,
                                         size, half_l, order);
     }, py::arg("f"), py::arg("phi"), py::arg("size"),
        py::arg("half_l")=false, py::arg("order")=AxisOrder::XYZ)
@@ -325,7 +343,7 @@ void add_hkl(py::module& m) {
                                  std::array<int, 3> size,
                                  bool half_l, AxisOrder order) {
         size_t col_idx = self.get_column_index(column);
-        return get_value_on_grid<float>(ReflnDataProxy{self}, col_idx,
+        return get_value_on_grid<float>(ReflnDataProxy(self), col_idx,
                                         size, half_l, order);
     }, py::arg("column"), py::arg("size"), py::arg("half_l")=false,
        py::arg("order")=AxisOrder::XYZ)
@@ -340,7 +358,7 @@ void add_hkl(py::module& m) {
         size_t phi_idx = self.get_column_index(phi_col);
         bool exact = (exact_size[0] != 0 || exact_size[1] != 0 ||
                       exact_size[2] != 0);
-        return transform_f_phi_to_map<float>(ReflnDataProxy{self},
+        return transform_f_phi_to_map<float>(ReflnDataProxy(self),
                                              f_idx, phi_idx,
                                              exact ? exact_size : min_size,
                                              sample_rate, exact, order);
@@ -367,4 +385,13 @@ void add_hkl(py::module& m) {
           r.first = cromer_libermann(z, energy, &r.second);
           return r;
         }, py::arg("z"), py::arg("energy"));
+  m.def("count_reflections", &count_reflections,
+        py::arg("cell"), py::arg("spacegroup"), py::arg("dmin"),
+        py::arg("dmax")=0., py::arg("unique")=true);
+  m.def("make_miller_array", [](const UnitCell& cell, const SpaceGroup* sg,
+                                double dmin, double dmax, bool unique) {
+          return py::array_t<int>(py::cast(
+                      gemmi::make_miller_vector(cell, sg, dmin, dmax, unique)));
+        }, py::arg("cell"), py::arg("spacegroup"), py::arg("dmin"),
+           py::arg("dmax")=0., py::arg("unique")=true);
 }
