@@ -7,7 +7,6 @@
 #include "gemmi/calculate.hpp"
 #include "gemmi/polyheur.hpp"
 #include "gemmi/assembly.hpp"
-#include "gemmi/labelseq.hpp"   // for setup_for_mmcif
 #include "gemmi/to_pdb.hpp"
 #include "gemmi/to_mmcif.hpp"
 #include "gemmi/tostr.hpp"
@@ -17,6 +16,7 @@
 #include <pybind11/pybind11.h>
 #include <pybind11/stl.h>
 #include <pybind11/stl_bind.h>
+#include "common.h"  // for normalize_index
 
 namespace py = pybind11;
 using namespace gemmi;
@@ -56,14 +56,6 @@ C& add_child(P& parent, C child, int pos) {
   return add_item(parent.children(), std::move(child), pos);
 }
 
-template<typename T> int normalize_index(int index, const T& container) {
-  if (index < 0)
-    index += (int) container.size();
-  if ((size_t) index >= container.size())
-    throw py::index_error();
-  return index;
-}
-
 template<typename P, typename C> C& get_child(P& parent, int index) {
   auto& children = parent.children();
   return children[normalize_index(index, children)];
@@ -265,7 +257,7 @@ void add_mol(py::module& m) {
                          bool seqres_records, bool ssbond_records,
                          bool link_records, bool cispep_records,
                          bool ter_records, bool numbered_ter,
-                         bool use_linkr) {
+                         bool ter_ignores_type, bool use_linkr) {
        PdbWriteOptions options;
        options.seqres_records = seqres_records;
        options.ssbond_records = ssbond_records;
@@ -273,6 +265,7 @@ void add_mol(py::module& m) {
        options.cispep_records = cispep_records;
        options.ter_records = ter_records;
        options.numbered_ter = numbered_ter;
+       options.ter_ignores_type = ter_ignores_type;
        options.use_linkr = use_linkr;
        Ofstream f(path);
        write_pdb(st, f.ref(), options);
@@ -280,7 +273,7 @@ void add_mol(py::module& m) {
        py::arg("seqres_records")=true, py::arg("ssbond_records")=true,
        py::arg("link_records")=true, py::arg("cispep_records")=true,
        py::arg("ter_records")=true, py::arg("numbered_ter")=true,
-       py::arg("use_linkr")=false)
+       py::arg("ter_ignores_type")=false, py::arg("use_linkr")=false)
     .def("write_minimal_pdb",
          [](const Structure& st, const std::string& path) {
        Ofstream f(path);
@@ -300,7 +293,7 @@ void add_mol(py::module& m) {
     .def("ensure_entities", &ensure_entities)
     .def("deduplicate_entities", &deduplicate_entities)
     .def("setup_entities", &setup_entities)
-    .def("setup_for_mmcif", &setup_for_mmcif)
+    .def("assign_label_seq_id", &assign_label_seq_id, py::arg("force")=false)
     .def("remove_alternative_conformations",
          remove_alternative_conformations<Structure>)
     .def("remove_hydrogens", remove_hydrogens<Structure>)
