@@ -6,8 +6,8 @@
 #include <array>
 #include <pybind11/pybind11.h>
 #include <pybind11/stl.h>
-//#include <pybind11/stl_bind.h>
-//#include <pybind11/numpy.h>
+#include <pybind11/numpy.h>
+#include "miller_a.h"
 
 namespace py = pybind11;
 using namespace gemmi;
@@ -32,11 +32,13 @@ template<typename T> void add_smat33(py::module& m, const char* name) {
     .def_readwrite("u12", &M::u12)
     .def_readwrite("u13", &M::u13)
     .def_readwrite("u23", &M::u23)
+    .def("elements", &M::elements)
     .def("as_mat33", &M::as_mat33)
     .def("trace", &M::trace)
     .def("nonzero", &M::nonzero)
     .def("determinant", &M::determinant)
     .def("inverse", &M::inverse)
+    .def("r_u_r", &M::r_u_r)
     .def("transformed_by", &M::transformed_by)
     .def("calculate_eigenvalues", &M::calculate_eigenvalues)
     ;
@@ -48,6 +50,8 @@ void add_unitcell(py::module& m) {
     .def_readwrite("x", &Vec3::x)
     .def_readwrite("y", &Vec3::y)
     .def_readwrite("z", &Vec3::z)
+    .def("dot", &Vec3::dot)
+    .def("cross", &Vec3::cross)
     .def("tolist", [](const Vec3& self) {
         return std::array<double,3>{{self.x, self.y, self.z}};
     })
@@ -67,6 +71,7 @@ void add_unitcell(py::module& m) {
     .def(py::init<>())
     .def("multiply", (Mat33 (Mat33::*)(const Mat33&) const) &Mat33::multiply)
     .def("multiply", (Vec3 (Mat33::*)(const Vec3&) const) &Mat33::multiply)
+    .def("left_multiply", &Mat33::left_multiply)
     .def("transpose", &Mat33::transpose)
     .def("approx", &Mat33::approx, py::arg("other"), py::arg("epsilon"))
     .def("determinant", &Mat33::determinant)
@@ -143,6 +148,10 @@ void add_unitcell(py::module& m) {
     .def_readonly("gamma", &UnitCell::gamma)
     .def_readonly("volume", &UnitCell::volume)
     .def_readonly("images", &UnitCell::images)
+    .def_property_readonly("fractionalization_matrix",
+                           [](const UnitCell& self) { return self.frac.mat; })
+    .def_property_readonly("orthogonalization_matrix",
+                           [](const UnitCell& self) { return self.orth.mat; })
     .def("set", &UnitCell::set)
     .def("is_crystal", &UnitCell::is_crystal)
     .def("fractionalize", &UnitCell::fractionalize)
@@ -158,10 +167,17 @@ void add_unitcell(py::module& m) {
          (int (UnitCell::*)(const Fractional&, double) const)
            &UnitCell::is_special_position,
          py::arg("fpos"), py::arg("max_dist"))
-    .def("calculate_1_d2",
-         (double (UnitCell::*)(const Miller&)const) &UnitCell::calculate_1_d2,
-         py::arg("hkl"))
+    .def("calculate_1_d2", &UnitCell::calculate_1_d2, py::arg("hkl"))
+    .def("calculate_1_d2_array", [](const UnitCell& u, py::array_t<int> hkl) {
+        return miller_function<double>(u, &UnitCell::calculate_1_d2, hkl);
+    })
     .def("calculate_d", &UnitCell::calculate_d, py::arg("hkl"))
+    .def("calculate_d_array", [](const UnitCell& u, py::array_t<int> hkl) {
+        return miller_function<double>(u, &UnitCell::calculate_d, hkl);
+    })
+    .def("metric_tensor", &UnitCell::metric_tensor)
+    .def("reciprocal_metric_tensor", &UnitCell::reciprocal_metric_tensor)
+    .def("get_hkl_limits", &UnitCell::get_hkl_limits, py::arg("dmin"))
     .def("__repr__", [](const UnitCell& self) {
         return "<gemmi.UnitCell(" + triple(self.a, self.b, self.c)
              + ", " + triple(self.alpha, self.beta, self.gamma) + ")>";
